@@ -1,5 +1,3 @@
-/* $XdotOrg: xc/programs/Xserver/hw/xfree86/common/xf86Config.c,v 1.8 2005/05/18 10:31:53 eich Exp $ */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Config.c,v 3.276 2003/10/08 14:58:26 dawes Exp $ */
 
 
 /*
@@ -67,6 +65,7 @@
 #include "xf86_OSlib.h"
 
 #include "globals.h"
+#include "extension.h"
 
 #ifdef XINPUT
 #include "xf86Xinput.h"
@@ -74,6 +73,7 @@ extern DeviceAssocRec mouse_assoc;
 #endif
 
 #ifdef XKB
+#undef XKB_IN_SERVER
 #define XKB_IN_SERVER
 #include <X11/extensions/XKBsrv.h>
 #endif
@@ -249,8 +249,9 @@ xf86ValidateFontPath(char *path)
 char **
 xf86ModulelistFromConfig(pointer **optlist)
 {
-    int count = 0;
+    int count = 0, i = 0;
     char **modulearray;
+    char *ignore[] = { "GLcore", "speedo", "bitmap", NULL };
     pointer *optarray;
     XF86LoadPtr modp;
     
@@ -271,7 +272,12 @@ xf86ModulelistFromConfig(pointer **optlist)
 	 */
 	modp = xf86configptr->conf_modules->mod_load_lst;
 	while (modp) {
-	    count++;
+            for (i = 0; ignore[i]; i++) {
+                if (strcmp(modp->load_name, ignore[i]) == 0)
+                    modp->ignore = 1;
+            }
+            if (!modp->ignore)
+	        count++;
 	    modp = (XF86LoadPtr) modp->list.next;
 	}
     }
@@ -287,9 +293,11 @@ xf86ModulelistFromConfig(pointer **optlist)
     if (xf86configptr->conf_modules) {
 	modp = xf86configptr->conf_modules->mod_load_lst;
 	while (modp) {
-	    modulearray[count] = modp->load_name;
-	    optarray[count] = modp->load_opt;
-	    count++;
+            if (!modp->ignore) {
+	        modulearray[count] = modp->load_name;
+	        optarray[count] = modp->load_opt;
+	        count++;
+            }
 	    modp = (XF86LoadPtr) modp->list.next;
 	}
     }
@@ -489,6 +497,7 @@ fixup_video_driver_list(char **drivers)
                 }
             }
             /* if we get here, ati was already ahead of atimisc */
+            return;
         }
     }
 }
@@ -502,50 +511,16 @@ fixup_video_driver_list(char **drivers)
 static char **
 GenerateDriverlist(char * dirname, char * drivernames)
 {
-#ifdef XFree86LOADER
     char **ret;
-    char *subdirs[] = { dirname, NULL };
+    const char *subdirs[] = { dirname, NULL };
     static const char *patlist[] = {"(.*)_drv\\.so", "(.*)_drv\\.o", NULL};
     ret = LoaderListDirs(subdirs, patlist);
     
     /* fix up the probe order for video drivers */
-    if (strstr(dirname, "drivers"))
+    if (strstr(dirname, "drivers") && ret != NULL)
         fixup_video_driver_list(ret);
 
     return ret;
-#else /* non-loadable server */
-    char *cp, **driverlist;
-    int count;
-
-    /* Count the number needed */
-    count = 0;
-    cp = drivernames;
-    while (*cp) {
-	while (*cp && isspace(*cp)) cp++;
-	if (!*cp) break;
-	count++;
-	while (*cp && !isspace(*cp)) cp++;
-    }
-
-    if (!count)
-	return NULL;
-
-    /* Now allocate the array of pointers to 0-terminated driver names */
-    driverlist = (char **)xnfalloc((count + 1) * sizeof(char *));
-    count = 0;
-    cp = drivernames;
-    while (*cp) {
-	while (*cp && isspace(*cp)) cp++;
-	if (!*cp) break;
-	driverlist[count++] = cp;
-	while (*cp && !isspace(*cp)) cp++;
-	if (!*cp) break;
-	*cp++ = 0;
-    }
-    driverlist[count] = NULL;
-
-    return driverlist;
-#endif
 }
 
 
@@ -607,6 +582,9 @@ static Bool
 configFiles(XF86ConfFilesPtr fileconf)
 {
   MessageType pathFrom = X_DEFAULT;
+  int countDirs;
+  char *temp_path;
+  char *log_buf;
 
   /* FontPath */
 
@@ -665,7 +643,36 @@ configFiles(XF86ConfFilesPtr fileconf)
   if (! *defaultFontPath)
     FatalError("No valid FontPath could be found.");
 
-  xf86Msg(pathFrom, "FontPath set to \"%s\"\n", defaultFontPath);
+  /* make fontpath more readable in the logfiles */
+  countDirs = 1;
+  temp_path = defaultFontPath;
+  while((temp_path = index(temp_path, ',')) != NULL) {
+    countDirs++;
+    temp_path++;
+  }
+  log_buf = xnfalloc(strlen(defaultFontPath) + (2 * countDirs) + 1);
+  if(!log_buf) /* fallback to old method */
+    xf86Msg(pathFrom, "FontPath set to \"%s\"\n", defaultFontPath);
+  else {
+    char *start, *end;
+    int size;
+    temp_path = log_buf;
+    start = defaultFontPath;
+    while((end = index(start, ',')) != NULL) {
+      size = (end - start) + 1;
+      *(temp_path++) = '\t';
+      strncpy(temp_path, start, size);
+      temp_path += size;
+      *(temp_path++) = '\n';
+      start += size;
+    }
+    /* copy last entry */
+    *(temp_path++) = '\t';
+    strcpy(temp_path, start);
+    xf86Msg(pathFrom, "FontPath set to:\n%s\n", log_buf);
+    xfree(log_buf);
+  }
+
 
   /* RgbPath */
 
@@ -689,7 +696,6 @@ configFiles(XF86ConfFilesPtr fileconf)
   }
   
   
-#ifdef XFree86LOADER
   /* ModulePath */
 
   if (fileconf) {
@@ -700,7 +706,6 @@ configFiles(XF86ConfFilesPtr fileconf)
   }
 
   xf86Msg(xf86ModPathFrom, "ModulePath set to \"%s\"\n", xf86ModulePath);
-#endif
 
 #if 0
   /* LogFile */
@@ -755,7 +760,8 @@ typedef enum {
     FLAG_LOG,
     FLAG_RENDER_COLORMAP_MODE,
     FLAG_HANDLE_SPECIAL_KEYS,
-    FLAG_RANDR
+    FLAG_RANDR,
+    FLAG_AIGLX
 } FlagValues;
    
 static OptionInfoRec FlagOptions[] = {
@@ -825,6 +831,8 @@ static OptionInfoRec FlagOptions[] = {
         {0}, FALSE },
   { FLAG_RANDR,			"RandR",			OPTV_BOOLEAN,
 	{0}, FALSE },
+  { FLAG_AIGLX,			"AIGLX",			OPTV_BOOLEAN,
+	{0}, FALSE },
   { -1,				NULL,				OPTV_NONE,
 	{0}, FALSE },
 };
@@ -1009,6 +1017,13 @@ configServerFlags(XF86ConfFlagsPtr flagsconf, XF86OptionPtr layoutopts)
     else
 	xf86Info.estimateSizesAggressively = 0;
 
+    xf86Info.aiglx = TRUE;
+    xf86Info.aiglxFrom = X_DEFAULT;
+    if (xf86GetOptValBool(FlagOptions, FLAG_AIGLX, &value)) {
+	xf86Info.aiglx = value;
+	xf86Info.aiglxFrom = X_CONFIG;
+    }
+
 /* Make sure that timers don't overflow CARD32's after multiplying */
 #define MAX_TIME_IN_MIN (0x7fffffff / MILLI_PER_MIN)
 
@@ -1283,7 +1298,7 @@ configInputKbd(IDevPtr inputp)
 
 #define NULL_IF_EMPTY(s) (s[0] ? s : (xfree(s), (char *)NULL))
 
-  if (!noXkbExtension && !XkbInitialMap) {
+  if (!noXkbExtension) {
     if ((s = xf86SetStrOption(inputp->commonOptions, "XkbKeymap", NULL))) {
       xf86Info.xkbkeymap = NULL_IF_EMPTY(s);
       xf86Msg(X_CONFIG, "XKB: keymap: \"%s\" "
@@ -1780,8 +1795,10 @@ configLayout(serverLayoutPtr servlayoutp, XF86ConfLayoutPtr conf_layout,
 	else
 	    scrnum = adjp->adj_scrnum;
 	if (!configScreen(slp[count].screen, adjp->adj_screen, scrnum,
-			  X_CONFIG))
+			  X_CONFIG)) {
+	    xfree(slp);
 	    return FALSE;
+	}
 	slp[count].x = adjp->adj_x;
 	slp[count].y = adjp->adj_y;
 	slp[count].refname = adjp->adj_refscreen;
@@ -1910,8 +1927,10 @@ configLayout(serverLayoutPtr servlayoutp, XF86ConfLayoutPtr conf_layout,
     idp = conf_layout->lay_inactive_lst;
     count = 0;
     while (idp) {
-	if (!configDevice(&gdp[count], idp->inactive_device, FALSE))
+	if (!configDevice(&gdp[count], idp->inactive_device, FALSE)) {
+	    xfree(gdp);
 	    return FALSE;
+	}
         count++;
         idp = (XF86ConfInactivePtr)idp->list.next;
     }
@@ -1933,8 +1952,10 @@ configLayout(serverLayoutPtr servlayoutp, XF86ConfLayoutPtr conf_layout,
     irp = conf_layout->lay_input_lst;
     count = 0;
     while (irp) {
-	if (!configInput(&indp[count], irp->iref_inputdev, X_CONFIG))
+	if (!configInput(&indp[count], irp->iref_inputdev, X_CONFIG)) {
+	    xfree(indp);
 	    return FALSE;
+	}
 	indp[count].extraOptions = irp->iref_option_lst;
         count++;
         irp = (XF86ConfInputrefPtr)irp->list.next;
@@ -1994,8 +2015,10 @@ configImpliedLayout(serverLayoutPtr servlayoutp, XF86ConfScreenPtr conf_screen)
     slp = xnfcalloc(1, 2 * sizeof(screenLayoutRec));
     slp[0].screen = xnfcalloc(1, sizeof(confScreenRec));
     slp[1].screen = NULL;
-    if (!configScreen(slp[0].screen, conf_screen, 0, from))
+    if (!configScreen(slp[0].screen, conf_screen, 0, from)) {
+	xfree(slp);
 	return FALSE;
+    }
     servlayoutp->id = "(implicit)";
     servlayoutp->screens = slp;
     servlayoutp->inactives = xnfcalloc(1, sizeof(GDevRec));
@@ -2138,6 +2161,17 @@ configScreen(confScreenPtr screenp, XF86ConfScreenPtr conf_screen, int scrnum,
     return TRUE;
 }
 
+typedef enum {
+    MON_REDUCEDBLANKING
+} MonitorValues;
+
+static OptionInfoRec MonitorOptions[] = {
+  { MON_REDUCEDBLANKING,      "ReducedBlanking",        OPTV_BOOLEAN,
+       {0}, FALSE },
+  { -1,                                NULL,                   OPTV_NONE,
+       {0}, FALSE },
+};
+
 static Bool
 configMonitor(MonPtr monitorp, XF86ConfMonitorPtr conf_monitor)
 {
@@ -2159,6 +2193,7 @@ configMonitor(MonPtr monitorp, XF86ConfMonitorPtr conf_monitor)
     monitorp->gamma = zeros;
     monitorp->widthmm = conf_monitor->mon_width;
     monitorp->heightmm = conf_monitor->mon_height;
+    monitorp->reducedblanking = FALSE;
     monitorp->options = conf_monitor->mon_option_lst;
 
     /*
@@ -2278,6 +2313,11 @@ configMonitor(MonPtr monitorp, XF86ConfMonitorPtr conf_monitor)
 	    return FALSE;
     }
 
+    /* Check wether this Monitor accepts Reduced Blanking modelines */
+    xf86ProcessOptions(-1, monitorp->options, MonitorOptions);
+
+    xf86GetOptValBool(MonitorOptions, MON_REDUCEDBLANKING,
+                      &monitorp->reducedblanking);
     return TRUE;
 }
 
@@ -2456,9 +2496,6 @@ configExtensions(XF86ConfExtensionsPtr conf_ext)
 {
     XF86OptionPtr o;
 
-    /* Extension enable/disable in miinitext.c */
-    extern Bool EnableDisableExtension(char *name, Bool enable);
-
     if (conf_ext && conf_ext->ext_option_lst) {
 	for (o = conf_ext->ext_option_lst; o; o = xf86NextOption(o)) {
 	    char *name   = xf86OptionName(o);
@@ -2492,6 +2529,7 @@ configExtensions(XF86ConfExtensionsPtr conf_ext)
 		xf86Msg(X_ERROR,
 			"%s is not a valid value for the Extension option\n",
 			val);
+		xfree(n);
 		return FALSE;
 	    }
 
@@ -2502,6 +2540,7 @@ configExtensions(XF86ConfExtensionsPtr conf_ext)
 		xf86Msg(X_WARNING, "Ignoring unrecognized extension \"%s\"\n",
                         name);
 	    }
+	    xfree(n);
 	}
     }
 
@@ -2672,7 +2711,7 @@ xf86HandleConfigFile(Bool autoconfig)
            scanptr = xf86ConfigLayout.screens->screen->device->busID;
     }
     if (scanptr) {
-       int bus, device, func, stroffset = 0;
+       int bus, device, func;
        if (strncmp(scanptr, "PCI:", 4) != 0) {
            xf86Msg(X_WARNING, "Bus types other than PCI not yet isolable.\n"
                               "\tIgnoring IsolateDevice option.\n");
