@@ -1,4 +1,3 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/bus/Pci.c,v 1.82 2004/01/16 15:39:04 tsi Exp $ */
 /*
  * Pci.c - New server PCI access functions
  *
@@ -238,6 +237,8 @@ static int readPciBios( PCITAG Tag, CARD8* tmp, ADDRESS hostbase,
 
 static int (*pciOSHandleBIOS)(PCITAG Tag, int basereg, unsigned char *buf, int len);
 
+int xf86MaxPciDevs = MAX_PCI_DEVICES;
+
 /*
  * Platform specific PCI function pointers.
  *
@@ -938,7 +939,7 @@ xf86scanpci(int flags)
     xf86MsgVerb(X_INFO, 2, "PCI: PCI scan (all values are in hex)\n");
 #endif
 
-    while (idx < MAX_PCI_DEVICES && tag != PCI_NOT_FOUND) {
+    while (idx < xf86MaxPciDevs && tag != PCI_NOT_FOUND) {
 	devp = xcalloc(1, sizeof(pciDevice));
 	if (!devp) {
 	    xf86Msg(X_ERROR,
@@ -1001,6 +1002,8 @@ xf86scanpci(int flags)
 #endif
 
 	pci_devp[idx++] = devp;
+        if (idx == xf86MaxPciDevs)
+            break;
 	tag = pciFindNext();
 
 #ifdef DEBUGPCI
@@ -1068,6 +1071,20 @@ xf86scanpci(int flags)
     return pci_devp;
 }
 
+pciConfigPtr
+xf86GetPciConfigFromTag(PCITAG Tag)
+{
+    pciConfigPtr pDev;
+    int i = 0;
+
+    for (i = 0 ; (pDev = pci_devp[i]) && i <= MAX_PCI_DEVICES; i++) {
+	if (Tag == pDev->tag)
+	    return pDev;
+    }
+
+    return NULL;	/* Bad data */
+}
+
 CARD32
 pciCheckForBrokenBase(PCITAG Tag,int basereg)
 {
@@ -1158,13 +1175,18 @@ handlePciBIOS( PCITAG Tag, int basereg, unsigned char * buf, int len )
 
 	/* if we use a mem base save it and move it out of the way */
 	if (b_reg >= 0 && b_reg <= 5) {
+	    memType emptybase;
 	    savebase = pciReadLong(Tag, PCI_MAP_REG_START+(b_reg<<2));
 	    xf86MsgVerb(X_INFO,5,"xf86ReadPciBios: modifying membase[%i]"
 			" for device %i:%i:%i\n", basereg,
 			(int)PCI_BUS_FROM_TAG(Tag), (int)PCI_DEV_FROM_TAG(Tag),
 			(int)PCI_FUNC_FROM_TAG(Tag));
+	    if (!(emptybase = getEmptyPciRange(Tag,b_reg))) {
+		xf86Msg(X_ERROR,"Cannot find empty range to map base to\n");
+		return 0;
+	    }
 	    pciWriteLong(Tag, PCI_MAP_REG_START + (b_reg << 2),
-			 (CARD32)~0);
+			 emptybase);
 	}
 	/* Set ROM base address and enable ROM address decoding */
 	pciWriteLong(Tag, PCI_MAP_ROM_REG, romaddr
