@@ -1,5 +1,3 @@
-/* $XdotOrg: xserver/xorg/mi/miinitext.c,v 1.31 2006/02/15 19:05:54 ajax Exp $ */
-/* $XFree86: xc/programs/Xserver/mi/miinitext.c,v 3.67 2003/01/12 02:44:27 dawes Exp $ */
 /***********************************************************
 
 Copyright 1987, 1998  The Open Group
@@ -46,7 +44,6 @@ ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
 ******************************************************************/
-/* $Xorg: miinitext.c,v 1.4 2001/02/09 02:05:21 xorgcvs Exp $ */
 
 #ifdef HAVE_DIX_CONFIG_H
 #include <dix-config.h>
@@ -85,6 +82,7 @@ SOFTWARE.
 #include "misc.h"
 #include "extension.h"
 #include "micmap.h"
+#include "globals.h"
 
 #if defined(QNX4) /* sleaze for Watcom on QNX4 ... */
 #undef GLXEXT
@@ -107,6 +105,7 @@ SOFTWARE.
 #undef XF86DRI
 #undef DPMSExtension
 #undef FONTCACHE
+#undef COMPOSITE
 #undef DAMAGE
 #undef XFIXES
 #undef XEVIE
@@ -125,9 +124,6 @@ extern Bool noBigReqExtension;
 #ifdef COMPOSITE
 extern Bool noCompositeExtension;
 #endif
-#ifdef DAMAGE
-extern Bool noDamageExtension;
-#endif
 #ifdef DBE
 extern Bool noDbeExtension;
 #endif
@@ -143,9 +139,6 @@ extern Bool noFontCacheExtension;
 #ifdef GLXEXT
 extern Bool noGlxExtension;
 #endif
-#ifdef LBX
-extern Bool noLbxExtension;
-#endif
 #ifdef SCREENSAVER
 extern Bool noScreenSaverExtension;
 #endif
@@ -241,10 +234,6 @@ typedef void (*InitExtension)(INITARGS);
 #ifdef XKB
 #include <X11/extensions/XKB.h>
 #endif
-#ifdef LBX
-#define _XLBX_SERVER_
-#include <X11/extensions/lbxstr.h>
-#endif
 #ifdef XPRINT
 #include <X11/extensions/Print.h>
 #endif
@@ -253,7 +242,7 @@ typedef void (*InitExtension)(INITARGS);
 #include <X11/extensions/Xagstr.h>
 #endif
 #ifdef XCSECURITY
-#define _SECURITY_SERVER
+#include "securitysrv.h"
 #include <X11/extensions/securstr.h>
 #endif
 #ifdef PANORAMIX
@@ -267,12 +256,6 @@ typedef void (*InitExtension)(INITARGS);
 #endif
 
 /* FIXME: this whole block of externs should be from the appropriate headers */
-#ifdef XTESTEXT1
-extern void XTestExtension1Init(INITARGS);
-#endif
-#ifdef SHAPE
-extern void ShapeExtensionInit(INITARGS);
-#endif
 #ifdef EVI
 extern void EVIExtensionInit(INITARGS);
 #endif
@@ -322,9 +305,6 @@ extern void XCMiscExtensionInit(INITARGS);
 #ifdef XRECORD
 extern void RecordExtensionInit(INITARGS);
 #endif
-#ifdef LBX
-extern void LbxExtensionInit(INITARGS);
-#endif
 #ifdef DBE
 extern void DbeExtensionInit(INITARGS);
 #endif
@@ -433,9 +413,6 @@ static ExtensionToggle ExtensionToggleList[] =
 #ifdef GLXEXT
     { "GLX", &noGlxExtension },
 #endif
-#ifdef LBX
-    { "LBX", &noLbxExtension },
-#endif
 #ifdef SCREENSAVER
     { "MIT-SCREEN-SAVER", &noScreenSaverExtension },
 #endif
@@ -550,9 +527,6 @@ InitExtensions(argc, argv)
   if (!noPanoramiXExtension) PanoramiXExtensionInit();
 # endif
 #endif
-#ifdef XTESTEXT1
-    if (!noTestExtensions) XTestExtension1Init();
-#endif
 #ifdef SHAPE
     if (!noShapeExtension) ShapeExtensionInit();
 #endif
@@ -604,9 +578,6 @@ InitExtensions(argc, argv)
 #ifdef XRECORD
     if (!noTestExtensions) RecordExtensionInit(); 
 #endif
-#ifdef LBX
-    if (!noLbxExtension) LbxExtensionInit();
-#endif
 #ifdef DBE
     if (!noDbeExtension) DbeExtensionInit();
 #endif
@@ -697,9 +668,6 @@ InitVisualWrap()
 #else /* XFree86LOADER */
 /* List of built-in (statically linked) extensions */
 static ExtensionModule staticExtensions[] = {
-#ifdef XTESTEXT1
-    { XTestExtension1Init, "XTEST1", &noTestExtensions, NULL, NULL },
-#endif
 #ifdef MITSHM
     { ShmExtensionInit, SHMNAME, &noMITShmExtension, NULL, NULL },
 #endif
@@ -715,9 +683,6 @@ static ExtensionModule staticExtensions[] = {
 #ifdef XKB
     { XkbExtensionInit, XkbName, &noXkbExtension, NULL, NULL },
 #endif
-#ifdef LBX
-    { LbxExtensionInit, LBXNAME, &noLbxExtension, NULL, NULL },
-#endif
 #ifdef XAPPGROUP
     { XagExtensionInit, XAGNAME, &noXagExtension, NULL, NULL },
 #endif
